﻿using System;
using System.Net.Sockets;

namespace EonaCat.Network
{
    // This file is part of the EonaCat project(s) which is released under the Apache License.
    // See the LICENSE file or go to https://EonaCat.com/License for full license details.

    public class TcpConnectedPeer
    {
        public string Token;

        /// <summary>
        /// The callback function when the message is received The return value is directly returned to the client. If null is returned, no reply
        /// </summary>
        public Func<string, byte[], byte[]> ResponseCallBack;

        /// <summary>
        /// Send a message to the client of this node
        /// </summary>
        /// <param name="message">message.</param>
        public void SendDataToClient(byte[] message)
        {
            try
            {
                this.SocketHandler.BeginSend(
                    message,
                    0,
                    message.Length,
                    SocketFlags.None,
                    new AsyncCallback(PeerSendCallBack), this.SocketHandler);
            }
            catch (Exception exception)
            {
                NetworkHelper.Logger.Exception(exception);
                SocketHandler.Close();
            }
        }

        public TcpServer Server { get; private set; }

        public Socket SocketHandler { get; private set; }

        public Action<Exception> OnDisconnected;

        public int BufferSize = 1024;
        private byte[] buffer;

        public TcpConnectedPeer(
            string token,
            Socket socket,
            Func<string, byte[], byte[]> OnReceived,
            TcpServer fromServer)
        {
            Token = token;
            SocketHandler = socket;
            ResponseCallBack = OnReceived;

            buffer = new byte[BufferSize];

            SocketHandler.BeginReceive(
                buffer,
                0,
                BufferSize,
                SocketFlags.None,
                new AsyncCallback(PeerReceiveCallBack),
                SocketHandler
            );
        }

        private void PeerReceiveCallBack(IAsyncResult ar)
        {
            Socket _clientHander = (Socket)ar.AsyncState;

            int byteLength = 0;

            byte[] receivedData;

            byteLength = _clientHander.EndReceive(ar);

            receivedData = buffer;

            buffer = null;

            buffer = new byte[BufferSize];

            if (byteLength > 0)
            {
                _clientHander.BeginReceive(
                buffer,
                0,
                BufferSize,
                SocketFlags.None,
                new AsyncCallback(PeerReceiveCallBack),
                _clientHander
                );
            }
            else
            {
                SocketHandler.Close();
                OnDisconnected?.Invoke(null);
                return;
            }

            try
            {
                if (ResponseCallBack != null)
                {
                    byte[] result = (ResponseCallBack(Token, receivedData));

                    if (result != null && result.Length > 0)
                    {
                        SendDataToClient(result);
                    }
                }
            }
            catch (Exception exception)
            {
                NetworkHelper.Logger.Exception(exception);
            }
        }

        private void PeerSendCallBack(IAsyncResult ar)
        {
            try
            {
                Socket handler = (Socket)ar.AsyncState;
                int SendBytesLength = handler.EndSend(ar);
            }
            catch (Exception e)
            {
                SocketHandler.Shutdown(SocketShutdown.Both);
                SocketHandler.Close();
                OnDisconnected?.Invoke(e);
            }
        }
    }
}