﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace EonaCat.Network
{
    // This file is part of the EonaCat project(s) which is released under the Apache License.
    // See the LICENSE file or go to https://EonaCat.com/License for full license details.

    public class Token
    {
        /// <summary>
        /// Randomly generate a unique Token string
        /// </summary>
        /// <returns>The generated token string.</returns>
        public static string GenerateGuid()
        {
            lock (_tokens)
            {
                string guid = Guid.NewGuid().ToString();
                if (_tokens.Keys.ToList().Contains(guid))
                {
                    return GenerateGuid();
                }
                return guid;
            }
        }

        public int Count => _tokens.Count;

        /// <summary>
        /// Each token saves a custom object
        /// </summary>
        /// <param name="token">Token.</param>
        public TcpConnectedPeer this[string token]
        {
            get
            {
                lock (_tokens)
                {
                    if (_tokens.Keys.Contains(token))
                    {
                        return _tokens[token];
                    }
                    else
                    {
                        return null;
                    }
                }
            }

            set
            {
                lock (_tokens)
                {
                    if (value != null)
                    {
                        _tokens[token] = value;
                    }
                    else
                        if (_tokens.Keys.Contains(token))
                    {
                        _tokens.Remove(token);
                    }
                }
            }
        }

        /// <summary>
        /// All the saved token data objects are cleared and the duplicate detection is also reset
        /// </summary>
        public void Clear()
        {
            _tokens.Clear();
        }

        private static readonly Dictionary<string, TcpConnectedPeer> _tokens = new Dictionary<string, TcpConnectedPeer>();

        private Token()
        { }

        private static Token instance;

        public static Token Instance
        {
            get
            {
                if (instance == null)
                {
                    instance = new Token();
                }

                return instance;
            }
        }
    }
}